# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2007 Johan Dahlin
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import overrides_hack
overrides_hack

from common import TestCase
import unittest
import sys

import gi
gi.require_version("Gst", "1.0")
from gi.repository import Gst
Gst.init(None)


class TestFraction(TestCase):
    def testConstructor(self):
        Gst.init(None)

        frac = Gst.Fraction(1, 2)
        self.assertEqual(frac.num, 1)
        self.assertEqual(frac.denom, 2)

        frac = Gst.Fraction(1)
        self.assertEqual(frac.num, 1)
        self.assertEqual(frac.denom, 1)

        self.assertRaises(TypeError, Gst.Fraction)

    def testRepr(self):
        Gst.init(None)

        self.assertEqual(repr(Gst.Fraction(1, 2)), '<Gst.Fraction 1/2>')

    def testEqNe(self):
        Gst.init(None)

        frac = Gst.Fraction(1, 2)
        self.assertEqual(frac, frac)
        self.assertEqual(Gst.Fraction(1, 2), Gst.Fraction(1, 2))
        self.assertEqual(Gst.Fraction(2, 4), Gst.Fraction(1, 2))

        self.assertNotEqual(Gst.Fraction(1, 3), Gst.Fraction(1, 2))
        self.assertNotEqual(Gst.Fraction(2, 1), Gst.Fraction(1, 2))

    def testMul(self):
        Gst.init(None)

        self.assertEqual(Gst.Fraction(1, 2) * Gst.Fraction(1, 2), Gst.Fraction(1, 4))
        self.assertEqual(Gst.Fraction(2, 3) * Gst.Fraction(4, 5), Gst.Fraction(8, 15))
        self.assertEqual(Gst.Fraction(1, 3) * Gst.Fraction(4), Gst.Fraction(4, 3))
        self.assertEqual(Gst.Fraction(1, 3) * 4, Gst.Fraction(4, 3))

    def testRMul(self):
        Gst.init(None)

        self.assertEqual(2 * Gst.Fraction(1, 2), Gst.Fraction(1))
        self.assertEqual(4 * Gst.Fraction(1, 2), Gst.Fraction(2))
        self.assertEqual(-10 * Gst.Fraction(1, 2), Gst.Fraction(-5))

    def testDiv(self):
        Gst.init(None)

        self.assertEqual(Gst.Fraction(1, 3) / Gst.Fraction(1, 4), Gst.Fraction(4, 3))
        self.assertEqual(Gst.Fraction(2, 3) / Gst.Fraction(4, 5), Gst.Fraction(10, 12))

        self.assertEqual(Gst.Fraction(1, 3) / Gst.Fraction(4), Gst.Fraction(1, 12))
        self.assertEqual(Gst.Fraction(1, 3) / 4, Gst.Fraction(1, 12))
        self.assertEqual(Gst.Fraction(1, 3) / 2, Gst.Fraction(1, 6))
        self.assertEqual(Gst.Fraction(1, 5) / -4, Gst.Fraction(1, -20))

    def testRDiv(self):
        Gst.init(None)

        self.assertEqual(2 / Gst.Fraction(1, 3), Gst.Fraction(6, 1))
        self.assertEqual(-4 / Gst.Fraction(1, 5), Gst.Fraction(-20, 1))

    def testFloat(self):
        Gst.init(None)

        self.assertEqual(float(Gst.Fraction(1, 2)), 0.5)

    def testPropertyMarshalling(self):
        Gst.init(None)

        obj = Gst.ElementFactory.make("rawvideoparse")
        if not obj:
            obj = Gst.ElementFactory.make("rawvideoparse")

        if not obj:
            # no (raw)videoparse and I don't know of any elements in core or -base using
            # fraction properties. Skip this test.
            return

        value = obj.props.framerate
        self.assertEqual(value.num, 25)
        self.assertEqual(value.denom, 1)

        obj.props.framerate = Gst.Fraction(2, 1)
        value = obj.props.framerate
        self.assertEqual(value.num, 2)
        self.assertEqual(value.denom, 1)

        def bad():
            obj.props.framerate = 1
        self.assertRaises(TypeError, bad)

        value = obj.props.framerate
        self.assertEqual(value.num, 2)
        self.assertEqual(value.denom, 1)

    def testGetFractionValue(self):
        Gst.init(None)

        st = Gst.Structure.from_string("video/x-raw,framerate=10/1")[0]
        value = st["framerate"]

        self.assertEqual(value.num, 10)
        self.assertEqual(value.denom, 1)


class TestFractionRange(TestCase):
    def testConstructor(self):
        Gst.init(None)

        r = Gst.FractionRange(Gst.Fraction(1, 30), Gst.Fraction(1, 2))
        self.assertEqual(r.start, Gst.Fraction(1, 30))
        self.assertEqual(r.stop, Gst.Fraction(1, 2))
        self.assertRaises(TypeError, Gst.FractionRange, Gst.Fraction(1, 2), Gst.Fraction(1, 30))
        self.assertRaises(TypeError, Gst.FractionRange, 2, Gst.Fraction(1, 2))
        self.assertRaises(TypeError, Gst.FractionRange, Gst.Fraction(1, 2), 2)
        self.assertRaises(TypeError, Gst.FractionRange)

    def testRepr(self):
        Gst.init(None)

        self.assertEqual(repr(Gst.FractionRange(Gst.Fraction(1, 30), Gst.Fraction(1, 2))),
                '<Gst.FractionRange [1/30,1/2]>')

    def testGetValue(self):
        Gst.init(None)

        st = Gst.Structure.new_empty("video/x-raw")
        st["range"] = Gst.FractionRange(Gst.Fraction(1, 30), Gst.Fraction(1, 2))
        value = st["range"]

        self.assertEqual(value.start, Gst.Fraction(1, 30))
        self.assertEqual(value.stop, Gst.Fraction(1, 2))


class TestDoubleRange(TestCase):
    def testConstructor(self):
        Gst.init(None)

        r = Gst.DoubleRange(1.2, 3.4)
        self.assertEqual(r.start, 1.2)
        self.assertEqual(r.stop, 3.4)
        self.assertRaises(TypeError, Gst.DoubleRange, {}, 2)
        self.assertRaises(TypeError, Gst.DoubleRange, 2, ())
        self.assertRaises(TypeError, Gst.DoubleRange, 2, 1)
        self.assertRaises(TypeError, Gst.DoubleRange)

    def testRepr(self):
        Gst.init(None)

        self.assertEqual(repr(Gst.DoubleRange(1, 2)), '<Gst.DoubleRange [1.0,2.0]>')

    def testGetValue(self):
        Gst.init(None)

        st = Gst.Structure.new_empty("video/x-raw")
        st["range"] = Gst.DoubleRange(1, 2)
        value = st["range"]

        self.assertEqual(value.start, 1.0)
        self.assertEqual(value.stop, 2.0)


class TestInt64Range(TestCase):
    @unittest.skipUnless(sys.version_info >= (3, 0), "requires Python 3")
    def testConstructor(self):
        Gst.init(None)

        r = Gst.Int64Range(range(0, 10, 2))
        self.assertEqual(r.range, range(0, 10, 2))
        self.assertRaises(TypeError, Gst.Int64Range, range(1, 10, 2))
        self.assertRaises(TypeError, Gst.Int64Range, range(0, 9, 2))
        self.assertRaises(TypeError, Gst.Int64Range, range(10, 0))
        self.assertRaises(TypeError, Gst.Int64Range, 1)
        self.assertRaises(TypeError, Gst.Int64Range)

    @unittest.skipUnless(sys.version_info >= (3, 0), "requires Python 3")
    def testRepr(self):
        Gst.init(None)

        self.assertEqual(repr(Gst.Int64Range(range(0, 10, 2))), '<Gst.Int64Range [0,10,2]>')

    @unittest.skipUnless(sys.version_info >= (3, 0), "requires Python 3")
    def testGetValue(self):
        Gst.init(None)

        st = Gst.Structure.new_empty("video/x-raw")
        st["range"] = Gst.Int64Range(range(0, 10, 2))
        value = st["range"]

        self.assertEqual(value, range(0, 10, 2))


class TestValueArray(TestCase):
    def testConstructor(self):
        Gst.init(None)

        a = Gst.ValueArray((1, 2, 3))
        self.assertEqual(a.array, [1, 2, 3])

        self.assertRaises(TypeError, Gst.ValueArray, 1)

    def testRepr(self):
        Gst.init(None)

        self.assertEqual(repr(Gst.ValueArray([1, 2, 3])), '<Gst.ValueArray <1,2,3>>')

    def testPropertyMarshalling(self):
        Gst.init(None)

        obj = Gst.ElementFactory.make("rawvideoparse")

        if not obj:
            # no rawvideoparse and I don't know of any elements in core or -base using
            # fraction properties. Skip this test.
            return

        value = obj.props.plane_strides
        self.assertEqual(value[0], 320)
        self.assertEqual(value[1], 160)
        self.assertEqual(value[2], 160)

        obj.props.plane_strides = Gst.ValueArray([640, 320, 320])

        value = obj.props.plane_strides
        self.assertEqual(value[0], 640)
        self.assertEqual(value[1], 320)
        self.assertEqual(value[2], 320)

        def bad():
            obj.props.plane_strides = 1
        self.assertRaises(TypeError, bad)

        value = obj.props.plane_strides
        self.assertEqual(value[0], 640)
        self.assertEqual(value[1], 320)
        self.assertEqual(value[2], 320)

    def testGetValue(self):
        Gst.init(None)

        st = Gst.Structure.new_empty("video/x-raw")
        st["array"] = Gst.ValueArray([Gst.Fraction(1, 30), Gst.Fraction(1, 2)])
        value = st["array"]
        st["array"] = Gst.ValueArray(value)

        self.assertEqual(value[0], Gst.Fraction(1, 30))
        self.assertEqual(value[1], Gst.Fraction(1, 2))

        st["matrix"] = Gst.ValueArray([Gst.ValueArray([0, 1]), Gst.ValueArray([-1, 0])])
        value = st["matrix"]

        self.assertEqual(value[0][0], 0)
        self.assertEqual(value[0][1], 1)
        self.assertEqual(value[1][0], -1)
        self.assertEqual(value[1][1], 0)

    def testAppendPrepend(self):
        array = Gst.ValueArray()
        Gst.ValueArray.append_value(array, 2)
        Gst.ValueArray.prepend_value(array, 1)
        array.append(3)
        array.prepend(0)
        self.assertEqual(len(array), 4)
        self.assertEqual(array[0], 0)
        self.assertEqual(array[1], 1)
        self.assertEqual(array[2], 2)
        self.assertEqual(array[3], 3)
        self.assertEqual(Gst.ValueArray.get_size(array), 4)
        self.assertEqual(Gst.ValueArray.get_value(array, 0), 0)
        self.assertEqual(Gst.ValueArray.get_value(array, 1), 1)
        self.assertEqual(Gst.ValueArray.get_value(array, 2), 2)
        self.assertEqual(Gst.ValueArray.get_value(array, 3), 3)

    def testIterator(self):
        array = Gst.ValueArray([1, 2, 3, 4, 5])
        result = [i for i in array]
        self.assertEqual(result, [1, 2, 3, 4, 5])


class TestValueList(TestCase):
    def testConstructor(self):
        Gst.init(None)

        a = Gst.ValueList((1, 2, 3))
        self.assertEqual(a.array, [1, 2, 3])

        self.assertRaises(TypeError, Gst.ValueList, 1)

    def testRepr(self):
        Gst.init(None)

        self.assertEqual(repr(Gst.ValueList([1, 2, 3])), '<Gst.ValueList {1,2,3}>')

    def testGetValue(self):
        Gst.init(None)

        st = Gst.Structure.new_empty("video/x-raw")
        st["framerate"] = Gst.ValueList([Gst.Fraction(1, 30), Gst.Fraction(1, 2)])
        value = st["framerate"]

        self.assertEqual(value[0], Gst.Fraction(1, 30))
        self.assertEqual(value[1], Gst.Fraction(1, 2))

        st["matrix"] = Gst.ValueList([Gst.ValueList([0, 1]), Gst.ValueList([-1, 0])])
        value = st["matrix"]

        self.assertEqual(value[0][0], 0)
        self.assertEqual(value[0][1], 1)
        self.assertEqual(value[1][0], -1)
        self.assertEqual(value[1][1], 0)

    def testAppendPrepend(self):
        array = Gst.ValueList()
        Gst.ValueList.append_value(array, 2)
        Gst.ValueList.prepend_value(array, 1)
        array.append(3)
        array.prepend(0)
        self.assertEqual(len(array), 4)
        self.assertEqual(array[0], 0)
        self.assertEqual(array[1], 1)
        self.assertEqual(array[2], 2)
        self.assertEqual(array[3], 3)
        self.assertEqual(Gst.ValueList.get_size(array), 4)
        self.assertEqual(Gst.ValueList.get_value(array, 0), 0)
        self.assertEqual(Gst.ValueList.get_value(array, 1), 1)
        self.assertEqual(Gst.ValueList.get_value(array, 2), 2)
        self.assertEqual(Gst.ValueList.get_value(array, 3), 3)


class TestIntRange(TestCase):
    @unittest.skipUnless(sys.version_info >= (3, 0), "requires Python 3")
    def testConstructor(self):
        Gst.init(None)

        r = Gst.IntRange(range(0, 10, 2))
        self.assertEqual(r.range, range(0, 10, 2))
        self.assertRaises(TypeError, Gst.IntRange, range(1, 10, 2))
        self.assertRaises(TypeError, Gst.IntRange, range(0, 9, 2))
        self.assertRaises(TypeError, Gst.IntRange, range(10, 0))
        self.assertRaises(TypeError, Gst.IntRange, 1)
        self.assertRaises(TypeError, Gst.IntRange)

    @unittest.skipUnless(sys.version_info >= (3, 0), "requires Python 3")
    def testRepr(self):
        Gst.init(None)

        self.assertEqual(repr(Gst.IntRange(range(0, 10, 2))), '<Gst.IntRange [0,10,2]>')

    @unittest.skipUnless(sys.version_info >= (3, 0), "requires Python 3")
    def testGetValue(self):
        Gst.init(None)

        st = Gst.Structure.new_empty("video/x-raw")
        st["range"] = Gst.IntRange(range(0, 10, 2))
        value = st["range"]

        self.assertEqual(value, range(0, 10, 2))


class TestBitmask(TestCase):
    def testConstructor(self):
        Gst.init(None)

        r = Gst.Bitmask(1 << 5)
        self.assertEqual(r, 1 << 5)

    def testGetValue(self):
        Gst.init(None)

        self.assertEqual(Gst.Structure('test,test=(bitmask)0x20')['test'], 1 << 5)

    def testStr(self):
        Gst.init(None)

        r = Gst.Bitmask(1 << 5)
        if sys.version_info >= (3, 0):
            self.assertEqual(str(r), '0x20')
        else:
            self.assertEqual(str(r), '0x20L')


class TestFloat(TestCase):
    def testSetProperty(self):
        ''' Test we can set the mix-matrix property on audioconvert

        It requires the Gst.Float class to force the conversion of python float
        to G_TYPE_FLOAT instead of G_TYPE_DOUBLE.
        '''
        Gst.init(None)

        audioconvert = Gst.ElementFactory.make("audioconvert")
        if not audioconvert:
            self.skipTest("audioconvert not available")

        row = Gst.ValueArray([Gst.Float(0.0)])
        matrix = Gst.ValueArray([row])
        audioconvert.set_property("mix-matrix", matrix)

        value = audioconvert.get_property("mix-matrix")
        self.assertEqual(len(value), 1)
        self.assertEqual(len(value[0]), 1)
        self.assertEqual(value[0][0], 0.0)



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# gst-python
# Copyright (C) 2005 Thomas Vander Stichele
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
# Boston, MA 02110-1301, USA.

import gst
import time

import gobject
#gobject.threads_init() # so we can safely receive signals from threads

count = 0

def on_message_application(cutter, message, loop):
    global count
    s = message.structure
    which = 'below'
    if s['above']: which = 'above'
    print "%s: %s threshold" % (gst.TIME_ARGS(s['timestamp']), which)
    if s['above']: count += 1
    if count > 2: loop.quit()

def main():
    type = 'async'
    loop = gobject.MainLoop()

    pipeline = gst.Pipeline("cutter")
    src = gst.element_factory_make("sinesrc", "src")
    cutter = gst.element_factory_make("cutter")
    cutter.set_property('threshold', 0.5)
    sink = gst.element_factory_make("fakesink", "sink")
    pipeline.add(src, cutter, sink)
    src.link(cutter)
    cutter.link(sink)

    control = gst.Controller(src, "volume")
    control.set_interpolation_mode("volume", gst.INTERPOLATE_LINEAR)

    control.set("volume", 0, 0.0)
    control.set("volume", 2 * gst.SECOND, 1.0)
    control.set("volume", 4 * gst.SECOND, 0.0)
    control.set("volume", 6 * gst.SECOND, 1.0)
    control.set("volume", 8 * gst.SECOND, 0.0)
    control.set("volume", 10 * gst.SECOND, 1.0)

    bus = pipeline.get_bus()

    if type == 'async':
        bus.add_signal_watch()
        bus.connect('message::element', on_message_application, loop)
    else:
        # FIXME: needs wrapping in gst-python
        bus.set_sync_handler(bus.sync_signal_handler)
        bus.connect('sync-message::element', on_message_application, loop)

    pipeline.set_state(gst.STATE_PLAYING)

    loop.run()

    pipeline.set_state(gst.STATE_NULL)

if __name__ == "__main__":
    main()



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Copyright (C) 2025 Netflix Inc.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

import overrides_hack
overrides_hack

from common import TestCase

import gi
gi.require_version("Gst", "1.0")
gi.require_version("GstApp", "1.0")
from gi.repository import Gst
from gi.repository import GstApp
Gst.init(None)


class TestAppSink(TestCase):

    def test_appsink_object(self):
        # Create an appsink
        appsink = Gst.ElementFactory.make("appsink", None)
        self.assertIsNotNone(appsink)
        self.assertTrue(isinstance(appsink, GstApp.AppSink))
        appsink.set_state(Gst.State.PLAYING)

        # Send an event to the appsink
        pad = appsink.get_static_pad("sink")
        caps = Gst.Caps("audio/x-raw")
        pad.send_event(Gst.Event.new_caps(caps))

        # Send a buffer to the appsink
        pad.chain(Gst.Buffer.new_wrapped([42]))

        # 1st object pulled must be the event
        event = appsink.pull_object()
        self.assertTrue(isinstance(event, Gst.Event))
        self.assertTrue(caps.is_equal(event.parse_caps()))

        # 2nd object pulled must be the buffer
        sample = appsink.pull_object()
        self.assertTrue(isinstance(sample, Gst.Sample))
        buf = sample.get_buffer()
        with buf.map(Gst.MapFlags.READ) as info:
            self.assertEqual(info.data[0], 42)

        appsink.set_state(Gst.State.NULL)



#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# gstfile.py
# (c) 2005 Edward Hervey <edward at fluendo dot com>
# Discovers and prints out multimedia information of files

# This example shows how to use gst-python:
# _ in an object-oriented way (Discoverer class)
# _ subclassing a gst.Pipeline
# _ and overidding existing methods (do_iterate())

import os
import sys

import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')

from gst.extend.discoverer import Discoverer

class GstFile:
    """
    Analyses one or more files and prints out the multimedia information of
    each file.
    """

    def __init__(self, files):
        self.files = files
        self.mainloop = gobject.MainLoop()
        self.current = None

    def run(self):
        gobject.idle_add(self._discover_one)
        self.mainloop.run()

    def _discovered(self, discoverer, ismedia):
        discoverer.print_info()
        self.current = None
        if len(self.files):
            print "\n"
        gobject.idle_add(self._discover_one)
        
    def _discover_one(self):
        if not len(self.files):
            gobject.idle_add(self.mainloop.quit)
            return False
        filename = self.files.pop(0)
        if not os.path.isfile(filename):
            gobject.idle_add(self._discover_one)
            return False
        print "Running on", filename
        # create a discoverer for that file
        self.current = Discoverer(filename)
        # connect a callback on the 'discovered' signal
        self.current.connect('discovered', self._discovered)
        # start the discovery
        self.current.discover()
        return False

def main(args):
    if len(args) < 2:
        print 'usage: %s files...' % args[0]
        return 2

    gstfile = GstFile(args[1:])
    gstfile.run()

if __name__ == '__main__':
    sys.exit(main(sys.argv))



# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# gst-python - Python bindings for GStreamer
# Copyright (C) 2002 David I. Lehn
# Copyright (C) 2004 Johan Dahlin
# Copyright (C) 2005 Edward Hervey
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA

from common import gst, unittest, TestCase, pygobject_2_13

import sys

# since I can't subclass gst.Element for some reason, I use a bin here
# it don't matter to Jesus
class TestElement(gst.Bin):
    def break_it_down(self):
        self.debug('Hammer Time')

class ElementTest(TestCase):
    name = 'fakesink'
    alias = 'sink'

    def testGoodConstructor(self):
        element = gst.element_factory_make(self.name, self.alias)
        assert element is not None, 'element is None'
        assert isinstance(element, gst.Element)
        assert element.get_name() == self.alias

## FIXME : Make a new test for state changes, using bus signals
        
## class FakeSinkTest(ElementTest):
##     FAKESINK_STATE_ERROR_NONE           = "0"
##     FAKESINK_STATE_ERROR_NULL_READY,    = "1"
##     FAKESINK_STATE_ERROR_READY_PAUSED,  = "2"
##     FAKESINK_STATE_ERROR_PAUSED_PLAYING = "3"
##     FAKESINK_STATE_ERROR_PLAYING_PAUSED = "4"
##     FAKESINK_STATE_ERROR_PAUSED_READY   = "5"
##     FAKESINK_STATE_ERROR_READY_NULL     = "6"

##     name = 'fakesink'
##     alias = 'sink'
##     def setUp(self):
##         ElementTest.setUp(self)
##         self.element = gst.element_factory_make('fakesink', 'sink')

##     def tearDown(self):
##         self.element.set_state(gst.STATE_NULL)
##         del self.element
##         ElementTest.tearDown(self)

##     def checkError(self, old_state, state, name):
##         assert self.element.get_state() == gst.STATE_NULL
##         assert self.element.set_state(old_state)
##         assert self.element.get_state() == old_state
##         self.element.set_property('state-error', name)
##         self.error = False
##         def error_cb(element, source, gerror, debug):
##             assert isinstance(element, gst.Element)
##             assert element == self.element
##             assert isinstance(source, gst.Element)
##             assert source == self.element
##             assert isinstance(gerror, gst.GError)
##             self.error = True
            
##         self.element.connect('error', error_cb)
##         self.element.set_state (state)
##         assert self.error, 'error not set'
##         #assert error_message.find('ERROR') != -1
        
##         self.element.get_state() == old_state, 'state changed'
        
##     def testStateErrorNullReady(self):
##         self.checkError(gst.STATE_NULL, gst.STATE_READY,
##                         self.FAKESINK_STATE_ERROR_NULL_READY)
        
##     def testStateErrorReadyPaused(self):
##         self.checkError(gst.STATE_READY, gst.STATE_PAUSED,
##                         self.FAKESINK_STATE_ERROR_READY_PAUSED)
        
##     def testStateErrorPausedPlaying(self):
##         self.checkError(gst.STATE_PAUSED, gst.STATE_PLAYING,
##                         self.FAKESINK_STATE_ERROR_PAUSED_PLAYING)        

##     def testStateErrorPlayingPaused(self):
##         self.checkError(gst.STATE_PLAYING, gst.STATE_PAUSED,
##                         self.FAKESINK_STATE_ERROR_PLAYING_PAUSED)
        
##     def testStateErrorPausedReady(self):
##         self.checkError(gst.STATE_PAUSED, gst.STATE_READY,
##                         self.FAKESINK_STATE_ERROR_PAUSED_READY)

##     def testStateErrorReadyNull(self):
##         self.checkError(gst.STATE_READY, gst.STATE_NULL,
##                         self.FAKESINK_STATE_ERROR_READY_NULL)

##     def checkStateChange(self, old, new):
##         def state_change_cb(element, old_s, new_s):
##             assert isinstance(element, gst.Element)
##             assert element == self.element
##             assert old_s == old
##             assert new_s == new
            
##         assert self.element.set_state(old)
##         assert self.element.get_state(0.0)[1] == old

## # FIXME: replace with messages
## #        self.element.connect('state-change', state_change_cb)

##         assert self.element.set_state(new)
##         assert self.element.get_state(0.0)[1] == new
        
##     def testStateChangeNullReady(self):
##         self.checkStateChange(gst.STATE_NULL, gst.STATE_READY)
        
##     def testStateChangeReadyPaused(self):
##         self.checkStateChange(gst.STATE_READY, gst.STATE_PAUSED)

##     def testStateChangePausedPlaying(self):
##         self.checkStateChange(gst.STATE_PAUSED, gst.STATE_PLAYING)
        
##     def testStateChangePlayingPaused(self):
##         self.checkStateChange(gst.STATE_PLAYING, gst.STATE_PAUSED)
        
##     def testStateChangePausedReady(self):
##         self.checkStateChange(gst.STATE_PAUSED, gst.STATE_READY)

##     def testStateChangeReadyNull(self):
##         self.checkStateChange(gst.STATE_READY, gst.STATE_NULL)

class NonExistentTest(ElementTest):
    name = 'this-element-does-not-exist'
    alias = 'no-alias'
    
    testGoodConstructor = lambda s: None
    testGoodConstructor2 = lambda s: None

class FileSrcTest(ElementTest):
    name = 'filesrc'
    alias = 'source'
    
class FileSinkTest(ElementTest):
    name = 'filesink'
    alias = 'sink'

class ElementName(TestCase):
    def testElementStateGetName(self):
        get_name = gst.element_state_get_name
        for state in ('NULL',
                      'READY',
                      'PLAYING',
                      'PAUSED'):
            name = 'STATE_' + state
            assert hasattr(gst, name)
            attr = getattr(gst, name)
            assert get_name(attr) == state
            
        assert get_name(gst.STATE_VOID_PENDING) == 'VOID_PENDING'
        assert get_name(-1) == 'UNKNOWN!(-1)'
        self.assertRaises(TypeError, get_name, '')
        
class QueryTest(TestCase):
    def setUp(self):
        TestCase.setUp(self)
        self.pipeline = gst.parse_launch('fakesrc name=source ! fakesink')
        self.assertEquals(self.pipeline.__gstrefcount__, 1)

        self.element = self.pipeline.get_by_name('source')
        self.assertEquals(self.pipeline.__gstrefcount__, 1)
        self.assertEquals(self.element.__gstrefcount__, 2)
        self.assertEquals(sys.getrefcount(self.element), pygobject_2_13 and 2 or 3)

    def tearDown(self):
        del self.pipeline
        del self.element
        TestCase.tearDown(self)
        
    def testQuery(self):
        gst.debug('querying fakesrc in FORMAT_BYTES')
        res = self.element.query_position(gst.FORMAT_BYTES)
        self.assertEquals(self.pipeline.__gstrefcount__, 1)
        self.assertEquals(sys.getrefcount(self.pipeline), pygobject_2_13 and 2 or 3)
        self.assertEquals(self.element.__gstrefcount__, 2)
        self.assertEquals(sys.getrefcount(self.element), pygobject_2_13 and 2 or 3)
        assert res
        assert res[0] == 0
        self.assertRaises(gst.QueryError, self.element.query_position,
            gst.FORMAT_TIME)
        self.gccollect()

class QueueTest(TestCase):
    def testConstruct(self):
        queue = gst.element_factory_make('queue')
        assert queue.get_name() == 'queue0'
        self.assertEquals(queue.__gstrefcount__, 1)

class DebugTest(TestCase):
    def testDebug(self):
        e = gst.element_factory_make('fakesrc')
        e.error('I am an error string')
        e.warning('I am a warning string')
        e.info('I am an info string')
        e.debug('I am a debug string')
        e.log('I am a log string')
        e.debug('I am a formatted %s %s' % ('log', 'string'))

    def testElementDebug(self):
        e = TestElement("testelement")
        e.set_property("name", "testelement")
        e.break_it_down()
        
class LinkTest(TestCase):
    def testLinkNoPads(self):
        src = gst.Bin()
        sink = gst.Bin()
        self.assertRaises(gst.LinkError, src.link, sink)

    def testLink(self):
        src = gst.element_factory_make('fakesrc')
        sink = gst.element_factory_make('fakesink')
        self.failUnless(src.link(sink))
        # FIXME: this unlink leaks, no idea why
        # src.unlink(sink)
        # print src.__gstrefcount__

    def testLinkPads(self):
        src = gst.element_factory_make('fakesrc')
        sink = gst.element_factory_make('fakesink')
        # print src.__gstrefcount__
        self.failUnless(src.link_pads("src", sink, "sink"))
        src.unlink_pads("src", sink, "sink")

    def testLinkFiltered(self):
        # a filtered link uses capsfilter and thus needs a bin
        bin = gst.Bin()
        src = gst.element_factory_make('fakesrc')
        sink = gst.element_factory_make('fakesink')
        bin.add(src, sink)
        caps = gst.caps_from_string("audio/x-raw-int")

        self.failUnless(src.link(sink, caps))

        # DANGER WILL.  src is not actually connected to sink, since
        # there's a capsfilter in the way.  What a leaky abstraction.
        # FIXME
        # src.unlink(sink)

        # instead, mess with pads directly
        pad = src.get_pad('src')
        pad.unlink(pad.get_peer())
        pad = sink.get_pad('sink')
        pad.get_peer().unlink(pad)

if __name__ == "__main__":
    unittest.main()
